require "libs.loveframes"

local consoleLines = {}
local consoleOn = false
console = {}

console.update = loveframes.update
console.mousepressed = loveframes.mousepressed
console.mousereleased = loveframes.mousereleased

console.keypressed = loveframes.keypressed
console.keyreleased = loveframes.keyreleased

local frame, consoleTab, watchesTab, list, wlist, button

function console.init()
	frame = loveframes.Create("frame")
	frame:SetName("Console")
	frame:SetSize(500, 370)
	frame:CenterWithinArea(0,0,love.graphics.getWidth(),love.graphics.getHeight())
	frame.OnClose = function(object)
		list:Clear()
		consoleOn = false
	end
	frame:SetSkin("Transp")

	local tab = loveframes.Create("tabs", frame)
	tab:SetPos(5, 30)
	tab:SetSize(490, 300)
	consoleTab = loveframes.Create("panel")
	tab:AddTab("Console",consoleTab)

	watchesTab = loveframes.Create("panel")
	tab:AddTab("Watches",watchesTab)


	list = loveframes.Create("list", consoleTab)
	list:SetPos(0, 0)
	list:SetSize(480, 265)
	list:SetPadding(5)
	list:SetSpacing(5)
	list:SetAutoScroll(true)
	list.Update = function(object)
		if #object.children > 500 then
			object:Clear()
		end
	end
	
	wlist = loveframes.Create("list", watchesTab)
	wlist:SetPos(0, 0)
	wlist:SetSize(480, 265)
	wlist:SetPadding(5)
	wlist:SetSpacing(5)
	wlist:SetAutoScroll(true)
	

	button = loveframes.Create("button", frame)
	button:SetPos(5, 340)
	button:SetSize(490, 25)
	button:SetText("Clear")
	button.OnClick = function(object2, x, y)
	    list:Clear()
	end
	button.Update = function(object)
		local displaytype = list:GetDisplayType()
        if #list.children > 0 then
        	object:SetText("Clear (" .. #list.children ..")")
        	object:SetEnabled(true)
        	object:SetClickable(true)
        else
        	object:SetText("Clear")
        	object:SetEnabled(false)
        	object:SetClickable(false)
        end
	end
end

function console.toggle()
	consoleOn = not consoleOn
end

function console.enable()
	consoleOn = true
	frame:SetVisible(true)
end

function console.disable()
	consoleOn = false
	frame:SetVisible(false)
end

function console.write(...)
	print(...)
	--if not consoleOn then return end
	local args = {...}
	local msg = ""
	for _,v in pairs(args) do
		local text = loveframes.Create("text")
		text:SetText(tostring(v))
		list:AddItem(text)
	end
end

function console.addWatch(_name,_func)
	if not (type(_func) == "function") then return end
	local entry = loveframes.Create("text")
	entry:SetText("Watch ".._name)
	entry.Update = function(object)
		local success, ret = pcall(_func)
		if success then
			object:SetText(_name..": "..tostring(ret))
		else
			object:SetText(_name..": error: "..tostring(ret))
		end
	end
	wlist:AddItem(entry)
	--table.insert(watches,{name = _name, func = _func})
end

function console.draw()
	if not consoleOn then return end
	loveframes.draw()
	--[[for offset,msg in ipairs(consoleLines) do
		love.graphics.print(msg,0,(offset-1)*10)
	end]]
	-- clean up string list
end

console.init()