require "console"
require "camera"
require "tilemap"

local function hex(number)
	return tonumber(""..number,16)
end


local refmap = {
{1,   2,   3,   4,   5,   6,   7,   8,   9,   10,  11,  12},
{13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24},
{25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36},
{37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48},
{49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60},
{61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72},
{73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84},
{85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96},
{97,  98,  99,  100, 101, 102, 103, 104, 105, 106, 107, 108},
{109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120},
{121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132},
{133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144},
{145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156},
}--]]

local collisionMap = {
15,0, 15,15,15,hex("12"),15,15,15,15,15,15,
15,0, 15,15,15,15,15,15,15,15,15,15,
15,0, 15,15,15,15,15,15,15,15,15,8,
15,0, 15,15,15,15,15,15,15,15,15,15,
15,0, 15,15,15,15,15,0 ,15,15,15,15,
15,1, 15,15,15,15,15,15,15,15,15,15,
15,1, 15,15,15,15,15,15,15,15,15,15,
15,1, 15,15,15,15,15,15,15,15,15,15,
15,15,15,15,15,15,15,8 ,15,hex("11"),15,15,
15,15,15,15,15,15,15,15,15,15,15,15,
15,15,15,15,15,15,15,15,15,15,15,15,
15,15,15,15,15,15,15,15,15,15,15,15,
15,1, 15,15,15,15,15,15,15,15,15,15,
}

local map = {
{},
{},
{},
{0,  0,  13,},
{0,  0,  0,  0,  0,  0,  106,36, 36},
{0,  13,  0,  0,  0,  106,84, 56, 56},
{1,  1,  1,  0,  106,84, 56, 56, 56},
{116,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,92},
}

local details = {
{},
{},
{},
{},
{0,},
{0, 0, 0, 0, 0, 0, 46, 26, },
{41, 0, 0, 0, 0, 0, 58, 0, },
{},
}

local gravity = 1000

local tile, camera
local imgTest


local player = {
	x = 0,
	y = 0,
	floorLevel = 0,
	ceilLevel = 0,
	curSpeedX = 0,
	curSpeedY = 0,
	image = love.graphics.newImage("tiles/tile0.png"),
	isGrounded = false,
	jump = function(self)
		-- check if grounded before jump
		if self.isGrounded then
			-- need to parametrize it
			self.curSpeedY = -500
		end
	end,
	draw = function(self)
		love.graphics.draw(self.image,math.floor(self.x),math.floor(self.y))
	end,
	
	-- I dont want to overload draw function to do "physics" things
	-- besides that, the draw function doesnt have dt
	update = function(self,dt)
		-- need to check before assign? I hope i'm not screwing anything :P
		local nextObstacle
		if self.curSpeedX > 0 then -- going right
			local upperHit = tile:GetNextCollision(self.x,self.y,"right")
			local lowerHit = tile:GetNextCollision(self.x,self.y + self.image:getHeight() - 1,"right")
			nextObstacle = upperHit < lowerHit and upperHit or lowerHit
			if nextObstacle <= (self.x + self.image:getWidth() + self.curSpeedX * dt) then
				self.x = nextObstacle - self.image:getWidth()
				self.curSpeedX = 0
			else
				self.x = self.x + self.curSpeedX * dt
			end
		else -- going left
			local upperHit = tile:GetNextCollision(self.x + self.image:getWidth(),self.y,"left")
			local lowerHit = tile:GetNextCollision(self.x + self.image:getWidth(),self.y + self.image:getHeight() - 1,"left")
			nextObstacle = upperHit > lowerHit and upperHit or lowerHit
			if nextObstacle >= (self.x + self.curSpeedX*dt) then
				self.x = nextObstacle
				self.curSpeedX = 0
			else
				self.x = self.x + self.curSpeedX * dt
			end
		end
		
		-- need to check all sprite base extension
		local leftFloor = tile:GetNextCollision(self.x,self.y,"down")
		local rightFloor = tile:GetNextCollision(self.x + self.image:getWidth() - 1,self.y,"down")
		self.floorLevel = leftFloor < rightFloor and leftFloor or rightFloor

		--console.write("floorLevel = "..self.floorLevel)

		local leftCeil = tile:GetNextCollision(self.x,self.y,"up")
		local rightCeil = tile:GetNextCollision(self.x + self.image:getWidth() - 1,self.y,"up")
		self.ceilLevel = leftCeil > rightCeil and leftCeil or rightCeil

		self.y = self.y + self.curSpeedY * dt
		if self.y >= (self.floorLevel - self.image:getHeight()) then
			if self.curSpeedY > 0 then
				self.y = self.floorLevel - self.image:getHeight()
				self.curSpeedY = 0
				self.isGrounded = true
			end
		elseif self.y < self.ceilLevel then
			if self.curSpeedY < 0 then
				self.y = self.ceilLevel
				self.curSpeedY = 0
			end
		else
			self.curSpeedY = self.curSpeedY + gravity * dt
			self.isGrounded = false
		end
		
	end,
}
console.addWatch("player X",function() return player.x end)
console.addWatch("player Y",function() return player.y end)
console.addWatch("grounded",function() return player.isGrounded end)
function love.load()
	console.write("console loaded")
	tile = Tilemap("tiles/tiles_spritesheet.png", 70, 70, 0, 0)
	imgTest = love.graphics.newImage("desert_bg.png")
	tile:AddMap(map)
	tile:AddMap(details)
	tile:SetCollisionMap(collisionMap)
	camera = Camera()
	--camera:scale(2) --broken
	camera:newLayer(0.25,function() love.graphics.draw(imgTest,0,tile.height - imgTest:getHeight()) end)
	camera:newLayer(1,function() tile:draw() player:draw() end)
	camera:setBounds(0,
					 0,
					 (tile.width > camera:getWidth() and (tile.width - camera:getWidth()) or 0),
					 (tile.height > camera:getHeight()) and (tile.height > camera:getHeight()) or 0)
	love.graphics.setBackgroundColor(34,51,129)
end

function love.draw()
	camera:draw()
	--love.graphics.print("Camera: "..math.floor(camera:getX())..", "..math.floor(camera:getY()),0,0)
	--love.graphics.print("MapSize: "..tile.width..", "..tile.height,0,10)
	console.draw()
end

function love.keypressed(key, unicode)
	console.keypressed(key,unicode)
	if key == 'f1' then console.toggle() end
	if key == 'escape' then love.event.quit() end
end

function love.keyreleased(key)
	console.keyreleased(key)
end

function love.mousepressed(x,y,button)
	console.mousepressed(x,y,button)
end

function love.mousereleased(x,y,button)
	console.mousereleased(x,y,button)
end


local charSpeed = 300
local timeout = 0
function love.update(dt)
	console.update(dt)
	player:update(dt)
	if timeout >= 0.1 then
		details[6][8] = details[6][8] == 26 and 38 or 26
		timeout = 0
	end
	player.curSpeedX = 0
	timeout = timeout + dt
	if love.keyboard.isDown("up") then
		--camera:move(0,-1)
		player:jump()
	end
	if love.keyboard.isDown("down") then
		--camera:move(0,1)
	end
	if love.keyboard.isDown("left") then
		--camera:move(-1,0)
		player.curSpeedX = -charSpeed
	end
	if love.keyboard.isDown("right") then
		--camera:move(1,0)
		player.curSpeedX = charSpeed
	end
	--keep camera centered
	camera:setPosition(player.x-camera:getWidth()/2 + player.image:getWidth()/2,player.y-camera:getHeight()/2 + player.image:getHeight()/2)
end