Tilemap = {}
Tilemap.__index = Tilemap
--[[ 
Creates a new tileset

tileset   = tileset image base path
tileSizeX = single tile X size
tileSizeY = single tile Y size
width     = tile count in X coordinate
height    = tile count in Y coordinate
xSize     = max tiles to draw in x coordinates
ySize     = max tiles to draw in y coordinates
]]
function Tilemap.new(tileset, tileSizeX, tileSizeY, width, height, xSize, ySize)
	local t = {
		x = 0, -- draw offset
		y = 0,
		_tiles = {}, -- tiles images
		w = width,  -- tile count in x
		h = height, -- and y
		map = {}, -- tile map
		_tileSizeX = tileSizeX,
		_tileSizeY = tileSizeY,
	}
	
	-- init tiles images
	local i = 0
	while true do
		if not love.filesystem.exists(tileset..i..".png") then break end
		table.insert(t._tiles,love.graphics.newImage(tileset..i..".png"))
		i = i+1
	end
	
	-- init tile mapping
	for y=1,height do
		t.map[y] = {}
		for x=1,width do
			t.map[y][x] = 0
		end
	end
	
	setmetatable(t,Tilemap)
	return t
end
setmetatable(Tilemap, {__call = function(_,...) return Tilemap.new(...) end})

-- changes current mapped tileset
function Tilemap:SetMap(mapTable)
	self.map = mapTable
end

function Tilemap:draw()
	for y=1, self.h do
		for x=1, self.w do
			if not (self.map[y][x] == 0) then
				width = self._tiles[self.map[y][x]]:getWidth()
				height = self._tiles[self.map[y][x]]:getHeight()
				xPos = self.x + width*(x-1)
				yPos = self.y + height*(y-1)
				if (xPos + width) > 0 and (yPos + height) > 0 then			
					love.graphics.draw(self._tiles[self.map[y][x]],
									   xPos, 
									   yPos)
				end
			end
		end
	end
end